/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014, 2015. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('rmps.app.pickers.internal.LiveSearchBox'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('rmps.app.pickers.internal', 'LiveSearchBoxMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$

(function(){

	/**
	 * Copy of com.ibm.team.gc.web.ui.picker.LiveSearchBox
	 * 
	 * A search box that will start a search after a slight delay in typing. There is no
	 * need to hit <Enter>. Shows a little magnifying glass that can also be clicked and
	 * looks like a search box visually.
	 *
	 * This used to be part of BasePicker, but was removed to be able to use it directly
	 * on the Search page. Due to needing to support being used in several places, it
	 * has a number of callback methods that let other widgets know about its lifecycle.
	 *
	 * When instatiating, you must provide a `doSearch` method to the constructor
	 * to actually run the search. The base method does nothing.
	 */
	dojo.declare(
		'rmps.app.pickers.internal.LiveSearchBox', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('rmps.app', 'pickers/internal/templates/LiveSearchBox.html'), //$NON-NLS-1$ //$NON-NLS-2$ 
			widgetsInTemplate: false,
			messages: dojo.i18n.getLocalization('rmps.app.pickers.internal', 'LiveSearchBoxMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			// Caller can set these in constructor.
			searchLabel: '', //$NON-NLS-1$
			searchTitle: '', //$NON-NLS-1$
			placeholder: '', //$NON-NLS-1$

			_setPlaceholderAttr: {
				node: '_searchBox', //$NON-NLS-1$
				type: 'attribute', //$NON-NLS-1$
				attribute: 'placeholder' //$NON-NLS-1$
			},

			/**
			 * This widget forwards key events to a delegate object.
			 * The only required method is doSearch. Others
			 * are only for interested widgets and may not apply, e.g.,
			 * if the results are not selectable.
			 *
			 * This is a stub delegate, but doesn't do anything except
			 * show documentation. There's no need to try and reuse this.
			 */
			delegate: {
				/**
				 * Callers must override this somehow to do the actual search.
				 * @param  {strign} searchString   the string to search for.
				 * @return {dojo.Deferred}	a Deferred that will
				 *     fire once the search results are available. The .callback
				 *     should return a list of hit objects, that the caller can
				 *     interpret how he wants.
				 */
				doSearch: function(searchString) {
					throw new Error('You need to override or set the doSearch method to actually do a search: ' + searchString); //$NON-NLS-1$
				},

				/**
				 * Called when a new search starts and any visible results should be cleared.
				 * Before new results are ready.
				 */
				clearResults: function() {},

				/**
				 * If the results are selectable, the user navigated with
				 * the keyboard and the selection should change.
				 */
				selectNextResult: function() {},
				selectPreviousResult: function() {},


				/**
				 * Is results are selected, the selected should be cleared.
				 * They do no need to be cleared.
				 */
				deselectResults: function() {},

				/**
				 * Called before a search start in case progress images need to be updated.
				 */
				willDoSearch: function() {},

				onEnter: function() {}
			},

			constructor: function(args, node)
			{
				this._searchId = dijit.getUniqueId('rmps-app-LiveSearchBox'); //$NON-NLS-1$
			},

			postCreate: function() {
				this.inherited(arguments);

			},

			startup: function() {
				this._searchBox.select();
			},

			_onSubmit: function(e) {
				dojo.stopEvent(e);
				clearTimeout(this._pendingSearch);
				delete this._pendingSearch;

				if (this.delegate.onEnter) {
					this.delegate.onEnter();
				}
				else {
					this.runSearch();
				}
			},

			_onFieldKeyUp: function(e) {
				e.stopPropagation();
				
				var k = e.keyCode;
				
				if (k === dojo.keys.UP_ARROW) {
					dojo.stopEvent(e);
					this._delegate('selectPreviousResult'); //$NON-NLS-1$
					return;
				}
				else if (k === dojo.keys.DOWN_ARROW) {
					dojo.stopEvent(e);
					this._delegate('selectNextResult'); //$NON-NLS-1$
					return;
				}

				if (this._searchBox.value === this._lastSearchString) return;

				this._delegate('deselectResults'); //$NON-NLS-1$
				window.clearTimeout(this._pendingSearch);
				this._pendingSearch = window.setTimeout(dojo.hitch(this, this.runSearch), 400);
			},

			_runSearchEvent: function(e) {
				e && dojo.stopEvent(e);
				delete this._pendingSearch;

				if (e && e.target == this._searchIcon) {
					this._searchBox.select();
				}

				this.runSearch();
			},

			/**
			 * Helper method to check if a method exists on the delegate
			 * and call the method of it doesn.
			 * @param  {string} methodName name of method to call.
			 * @param  {object} arg1       any optional argument to pass to the method.
			 */
			_delegate: function(methodName, arg1) {
				if (this.delegate[methodName]) {
					this.delegate[methodName](arg1);
				}
			},

			/**
			 * Starts an actual search and asks the delegate to do it.
			 */
			runSearch: function() {
				clearTimeout(this._pendingSearch);

				this._delegate('willDoSearch'); //$NON-NLS-1$

				var searchString = dojo.trim(this._searchBox.value);
				this._lastSearchString = searchString;

				if (searchString.length === 0) {
					this._delegate('clearResults'); //$NON-NLS-1$
					return;
				}

				// ask subclass to really run the search
				this.delegate.doSearch(searchString);
			},

			/**
			 * Focuses on the search input box. Optionally overwrites the search
			 * string with a new one a runs that.
			 * @param  {string} optionalNewSearchString null to just focus.
			 */
			select: function(optionalNewSearchString) {
				this._searchBox.select();
				
				if (optionalNewSearchString) {
					this._searchBox.value = optionalNewSearchString;
				}
			},

			getSearchString: function() {
				return this._searchBox.value;
			},
			
			setSearchString: function(searchString) {
				this._searchBox.value = searchString;
				if (searchString.length === 0) {
					this._lastSearchString = "";
				}
			}
		}
	);

})();